# a_generate_INGdata+mfccs_from_aligned_CORAAL.R
# Kendall et al. Performance in Coding Sociolinguistic Variables (Frontiers)
#
# This code extracts instances of variable (ING) words along with other
#  word final -in and -ing words from the output of the Montreal Forced Aligner. 
#
# It uses tuneR package to extract MFCCs for the -in and -ing endings.
#
# It then looks up the variable (ING) words in CORAAL's regular transcripts
#  to extract information about the utterance for each (ING) word.
#
# The output from MFA, trained to differentiate -in and -ing in -ing final
#  words is included in the Supplementary Materials, but you will need to
#  downloaded the corresponding audio files from the CORAAL website
#  https://oraal.uoregon.edu/coraal/ before you can use this code.

# Load tuneR library - install it first if you don't have it (install.packages(...))
library(tuneR)

# Set the working folder, which is where the output will be saved 
wk.dir <- "~/Documents/CORAAL/ING/Supplementary Materials/"

# Find paths to all of the audio directories for CORAAL
#  This looks for folders matching the specific pattern - you can change or set the
#  the pattern, or specify the directory in some other way (i.e. update to match
#  your path) 
audio.dirs <- dir("~/Documents/CORAAL", pattern="audio_part\\d\\d_2018\\.10\\.06$", full.names=T, include.dirs=T)

# Now find all the wav files in the audio directors - tuneR will need these
all.wavs <- c()
for (ad in audio.dirs) {
	all.wavs <- c(all.wavs, dir(ad, pattern="wav$", full.names=T))
}
# Will need to be able to "look up" each wav based on its TextGrid name
lup.wavs <- gsub("^.*/(\\w\\w\\w_se\\d.*)\\.wav", "\\1", all.wavs, perl=T)

# CORAAL_Aligned_as_Tables contains MFA output files from the regular alignment for
#  CORAAL, i.e. not modeling different productions for (ING). 
#  The files were created from the MFA output TextGrids by batch_convert_to_table.praat
all.TGs <- dir(paste(wk.dir, "CORAAL_Aligned_as_Tables", sep=""), pattern="_txout\\.txt$", full.names=T)

dc.TGs <- grep("^.*/DC[AB]_", all.TGs, perl=T, value=T)

master.dat <- NA
first <- TRUE
for (TG in dc.TGs) {
	
	tgt <- gsub("^.*/(\\w\\w\\w_se\\d.*)_txout\\.txt", "\\1", TG, perl=T)
	wav <- all.wavs[which(lup.wavs==tgt)]
	
	cat("\nProcessing *", tgt, "*...", sep="")
	
	if (length(wav)!=1) {
		cat("\n  Could not match one wav to ", tgt, "; found ", length(wav), " instead. Skipping.", sep="")
		next
	} 
	
	tin <- read.delim(TG)

	stin <- droplevels(tin[-which(tin$text %in% c("sil", "sp", "TN", "rdx")),])
	
	stin$wav <- basename(wav)
	stin$spkr <- gsub(" - (phones|words)", "", stin$tier, perl=T)
	stin$type <- gsub("^.* - (.*?)s", "\\1", stin$tier, perl=T)
		
	wrd.locs <- which(stin$type=="word")
	inng.locs <- grep("ing?s?$", stin$text, perl=T)
		
	nxt.locs <- wrd.locs[which(wrd.locs %in% inng.locs)+1]
		
	inng.df <- NA
	innfirst <- TRUE
	for (i in 1:length(inng.locs)) {
		st <- inng.locs[i] - 1
		en <- nxt.locs[i] - 1
		
		phone.indices <- which((stin$type=="phone") & (stin$tmin >= stin$tmin[inng.locs[i]]) & (stin$tmin < stin$tmax[inng.locs[i]]))
		
		# This includes -ING words with final -s
		# Adjust phone indices for the cases where this is a final -s		
		s.offset <- ifelse(length(grep(".*s$", stin$text[inng.locs[i]], perl=T))==1, 1, 0)
		
		vowel.index <- phone.indices[length(phone.indices) - 1 - s.offset]
		nasal.index <- phone.indices[length(phone.indices) - s.offset]
		
		inng.tmp <- data.frame(
			spkr = unique(stin$spkr[phone.indices]),
			file = unique(stin$wav[phone.indices]),
			word = stin$text[inng.locs[i]],
			st.t = stin$tmin[inng.locs[i]],
			en.t = stin$tmax[inng.locs[i]],
			phns = paste(stin$text[phone.indices], collapse="|"),
			vwl.st = stin$tmin[vowel.index],
			vwl.en = stin$tmax[vowel.index],
			nas.st = stin$tmin[nasal.index],
			nas.en = stin$tmax[nasal.index]
			)
		inng.tmp$inng.dur <- inng.tmp$nas.en - inng.tmp$vwl.st
		
		# This matches multiple utterances during overlapping speech. 
		#  In this case, take just the first match. This is a known issue
		#  with the script and could likely be improved.
		if (nrow(inng.tmp) > 1) {
			cat("\n Matched ", nrow(inng.tmp), " utterances (likely overlapping speech) - only taking first match.", sep="")
			inng.tmp <- droplevels(inng.tmp[1,])
		}
			
		cat("\n Setting up word: ", as.character(inng.tmp$word), " (", as.character(inng.tmp$phns), ")", sep="")
		if (s.offset==1) {
			cat(" [ adjusted phone indices for final -s ]")
		}
		
		if (innfirst) {
			inng.df <- inng.tmp
			innfirst <- FALSE
		} else {
			inng.df <- rbind(inng.df, inng.tmp)
		}
	}
	# Drop words like pain and gain
	pain.words <- grep("[aeiou]ins?$", inng.df$word, perl=T)
	if (length(pain.words)>0) {
		inng.df <- droplevels(inng.df[-pain.words,])
	}
		
	# Code the words into three classes VAR (=variable context), 
	#  ING (=non-variable -ing words, like "thing" and "sting"),
	#  and IN (=non-variable -in words, like "thin" and "begin"; includes "in")
	# Note this may over-include some words; prudent to check the output in the end
	#  We ended up dropping the final -s words for non-variable categories
	inng.df$Class <- "VAR"
	inng.df$Class[grep("^[^aeiouy]+?ings?$", inng.df$word, perl=T)] <- "ING"
	inng.df$Class[grep("ins?$", inng.df$word, perl=T)] <- "IN"
	
	cat("\n *Stored: ", length(which(inng.df$Class=="VAR")), " variable words, ", length(which(inng.df$Class=="IN")), " IN words, and ", length(which(inng.df$Class=="ING")), " monosyllabic ING words", sep="")
	
	cat("\n Now generating MFCCs", sep="")
	mfcc.matr <- NA
	ccfirst <- TRUE
	for (i in 1:nrow(inng.df)) {
		
		mfcc.tmp <- NA
		ccrow <- rep(NA, times=12*4)
		
		# Process MFCCs only if the IN(G) is 50 ms or longer
		if (inng.df$inng.dur[i] >= 0.05) {
			win.tmp <- readWave(wav, from=inng.df$vwl.st[i], to=inng.df$nas.en[i], units="seconds")
			
			# We explored different settings for the MFCCs and settled on this
			#  but one might get better performance with different settings
			mfcc.tmp <- melfcc(win.tmp, numcep=12, wintime=0.02, hoptime=0.01, minfreq=100, maxfreq=10000)

			# Ideally one might take four time slices, 1/2 vowel, start of nas, 
			#  1/3 nas, 2/3 nas, but hard to identify exact point 
			#  rows are 0.01 sec apart, so we take rows that are 25%, 50%, 70%, 90%
			# But skip this if there are not at least 6 rows in the mfcc.tmp df
			# One might also get better perforamce with different time points for
			#  MFCCs or more (or fewer) MFCCs.
			if (nrow(mfcc.tmp) >= 6) {
				inds <- round(c(0.25, 0.50, 0.70, 0.90)*nrow(mfcc.tmp)) 
				ccrow <- sapply(t(mfcc.tmp[inds,]), "[")
			}
		} 
		fullrow <- c(i, ccrow)
		names(fullrow) <- c("TokNum", sapply(1:12, paste, "CepCoef_25", sep="_"), sapply(1:12, paste, "CepCoef_50", sep="."), sapply(1:12, paste, "CepCoef_70", sep="."), sapply(1:12, paste, "CepCoef_90", sep="."))
		
		if (ccfirst) {
			mfcc.matr <- fullrow
			ccfirst <- FALSE
		} else {
			mfcc.matr <- rbind(mfcc.matr, fullrow)
		}
	}
	rownames(mfcc.matr) <- mfcc.matr[,1]
	
	comb.dat <- cbind(inng.df, mfcc.matr)

	if (first) {
		master.dat <- comb.dat
		first <- FALSE
	} else {
		master.dat <- rbind(master.dat, comb.dat)
	}
	cat(" ...Completed.\n", sep="")
}

write.table(master.dat, file=paste(wk.dir, "YOUR_IN+ING_Words_wMFCCs.txt", sep=""), append=F, quote=T, sep="\t", row.names=F)

var.dat <- droplevels(master.dat[master.dat$Class=="VAR",])
var.dat$file <- gsub("\\.wav$", "", var.dat$file, perl=T)

# Load the CORAAL Explorer R code from its website
source("http://lingtools.uoregon.edu/coraal/explorer/R/CORAAL_web.R")

# Download CORAAL:DC transcripts
cdc <- coraal.webget.data("DC", turns=F, no.pauses=T)

# Now loop through the var.dat data and attach additional information 
#  from the main versions of the corpus transcripts
# Nb. there are much more efficient ways to do this in R, but this is easier
#  to follow and modify for many people
var.dat$Utter <- NA
var.dat$TLine <- NA
for (i in 1:nrow(var.dat)) {
	utt.ind <- which(cdc$File==as.character(var.dat$file[i]) & cdc$Spkr==as.character(var.dat$spkr[i]) & cdc$StTime <= var.dat$en.t[i] & cdc$EnTime >= var.dat$st.t[i])
	if (length(utt.ind)==1) {
		var.dat$Utter[i] <- as.character(cdc$Content[utt.ind])
		var.dat$TLine[i] <- cdc$Line[utt.ind]
	} else {
		cat("\nFailed to find a single match for '", as.character(var.dat$word[i]), "' by ", as.character(var.dat$spkr[i]), " in file ", as.character(var.dat$file[i]), " (matched ", length(utt.ind), ")", sep="")
	}
}
dim(var.dat)

# You can use these tokens for hand-coding variable (ING) (or other purposes)
write.table(var.dat, file=paste(wk.dir, "YOUR_VarIN_Tokens_wMFCCs.txt", sep=""), append=F, quote=T, sep="\t", row.names=F)
