# b_setup_handcoded_data_with_mfccs.R
# Kendall et al. Performance in Coding Sociolinguistic Variables (Frontiers)
#
# Code provides some initial exploration of the hand-coded data (Datasets A, B, & C)
#  and pre-processing. Models I & II from paper, and other table material are
#  generated here.
#
# Code as is will merge Datasets B & C with the extracted MFCCs used in the paper, but
#  you can replace our MFCC dataset with others created with the
#  a_generate_INGdata+mfccs_from_aligned_CORAAL.R code

# Set the working folder (where files reside and will be saved)
wk.dir <- "~/Documents/CORAAL/ING/Supplementary Materials/"

# Read in the initial data
fdat <- read.delim(paste(wk.dir, "DatasetsA+B_Initial.txt", sep=""))

# Examine speakers (FYI)
sps <- unique(fdat$Spkr)
scrps <- gsub("^(\\w{3}).*", "\\1", sps, perl=T)
sgens <- gsub("^.*?_([mf])_\\d\\d$", "\\1", sps, perl=T)
table(sgens)
sages <- gsub("^.*?_(ag\\d)_.*$", "\\1", sps, perl=T)
table(scrps, sages)
ssess <- gsub("^.*?_(se\\d)_.*$", "\\1", sps, perl=T)
table(scrps, ssess)

dim(fdat)

# Can examine the metadata from CORAAL, but we'll end up just grabbing what we
#  need from the speaker labels later
source('http://lingtools.uoregon.edu/coraal/explorer/R/CORAAL_web.R')
meta <- coraal.webget.meta("DC")
sp.meta.f <- droplevels(meta[meta$CORAAL.Spkr %in% sps,])
sp.meta <- droplevels(unique(sp.meta.f[,c(1,4,11:40)]))
dim(sp.meta)
table(sp.meta$CORAAL.Sub, sp.meta$Age.Group)

# Trim out any lines than any coder said don't count for
tdat <- droplevels(fdat[-grep("DC", paste(fdat$Coder1.NorG, fdat$Coder2.NorG, fdat$Coder3.NorG, fdat$Coder4.NorG, fdat$Coder5.NorG, fdat$Coder6.NorG, fdat$Coder7.NorG, sep=","), perl=T),])
dim(tdat)

# Convert the individual codes to a numerical system 
#  This is used for determining majority coding later, but also useful for examining gradience
table(tdat$Coder1.NorG)
tdat$Coder1.V <- ifelse(tdat$Coder1.NorG=="N", 0, ifelse(tdat$Coder1.NorG=="G", 1, ifelse(tdat$Coder1.NorG=="G?", 0.75, ifelse(tdat$Coder1.NorG=="N?", 0.25, NA))))

table(tdat$Coder2.NorG)
tdat$Coder2.V <- ifelse(tdat$Coder2.NorG=="N", 0, ifelse(tdat$Coder2.NorG=="G", 1, ifelse(tdat$Coder2.NorG=="G?", 0.75, ifelse(tdat$Coder2.NorG=="N?", 0.25, NA))))

table(as.character(tdat$Coder3.NorG))
tdat$Coder3.V <- ifelse(tdat$Coder3.NorG=="N", 0, ifelse(tdat$Coder3.NorG=="G", 1, ifelse(tdat$Coder3.NorG=="G?", 0.75, ifelse(tdat$Coder3.NorG=="N?", 0.25, NA))))
table(tdat$Coder3.V, useNA="ifany")

table(as.character(tdat$Coder4.NorG))
tdat$Coder4.V <- ifelse(tdat$Coder4.NorG=="N", 0, ifelse(tdat$Coder4.NorG=="G", 1, ifelse(tdat$Coder4.NorG=="G?", 0.75, ifelse(tdat$Coder4.NorG=="N?", 0.25, NA))))
table(tdat$Coder4.V, useNA="ifany")

table(as.character(tdat$Coder5.NorG))
tdat$Coder5.V <- ifelse(tdat$Coder5.NorG=="N", 0, ifelse(tdat$Coder5.NorG=="G", 1, ifelse(tdat$Coder5.NorG=="G?", 0.75, ifelse(tdat$Coder5.NorG=="N?", 0.25, NA))))
table(tdat$Coder5.V, useNA="ifany")

table(as.character(tdat$Coder6.NorG))
tdat$Coder6.V <- ifelse(tdat$Coder6.NorG=="N", 0, ifelse(tdat$Coder6.NorG=="G", 1, ifelse(tdat$Coder6.NorG=="G?", 0.75, ifelse(tdat$Coder6.NorG=="N?", 0.25, NA))))
table(tdat$Coder6.V, useNA="ifany")

table(as.character(tdat$Coder7.NorG))
tdat$Coder7.V <- ifelse(tdat$Coder7.NorG=="N", 0, ifelse(tdat$Coder7.NorG=="G", 1, ifelse(tdat$Coder7.NorG=="G?", 0.75, ifelse(tdat$Coder7.NorG=="N?", 0.25, NA))))
table(tdat$Coder7.V, useNA="ifany")

tdat$COMB.V <- rowMeans(tdat[, c("Coder1.V", "Coder2.V", "Coder3.V", "Coder4.V", "Coder5.V", "Coder6.V", "Coder7.V")], na.rm=T)

# Now separate out Dataset A (7 coders) and Dataset B (3 coders)
dataA <- droplevels(tdat[tdat$Dataset=="A",])
dim(dataA)
dataB <- droplevels(tdat[tdat$Dataset=="B",])
dim(dataB)

# Set the majority rules code 
#  For Dataset A, base this on conversion from the quantitative rating
dataA$MAJCODE <- ifelse(dataA$COMB.V < 0.5, "N", "G")
table(dataA$Spkr, dataA$MAJCODE)

# FOR TABLE 1 IN PAPER:
round(100*prop.table(table(dataA$Spkr, dataA$MAJCODE), 1), 1)

# For Dataset B, make it easier to work with the three codes
dataB$RAT1 <- dataB$Coder1.NorG
dataB$RAT2 <- dataB$Coder2.NorG
dataB$RATX <- NA
for (i in 1:nrow(dataB)) {
	row <- dataB[i,]
	rat.ind <- which(!is.na(row[,c("Coder3.V", "Coder4.V", "Coder5.V", "Coder6.V", "Coder7.V")]))
	dataB$RATX[i] <- as.character(row[,which(names(dataB)=="Coder3.NorG")-1+rat.ind])
}

# Make MFA output comparable to human codes
levels(dataB$MFA.NorG) <- c("N", "G")

# Explore Dataset B, with and without low confidence marks
dataBc <- dataB

# First though assess with low confidence marks (i.e. ?s)
stringof3qcodes <- rep(NA, times=nrow(dataBc))
count3qs <- 0
for (i in 1:nrow(dataBc)) {
	row <- t(dataBc[i,c("RAT1", "RAT2", "RATX")])
	temp <- sort(as.vector(row))
	if (length(grep("\\?", temp, perl=T))>0) count3qs <- count3qs + 1
	stringof3qcodes[i] <- paste(temp, collapse="-")
}
count3qs

# FOR TABLE 3 IN PAPER:
table(stringof3qcodes)
round(100*prop.table(table(stringof3qcodes)), 1)

# Now a version where they are removed, which we will go on to statistically model
dataBnc <- dataB
dataBnc$RAT1 <- gsub("\\?", "", dataBnc$RAT1, perl=T)
dataBnc$RAT2 <- gsub("\\?", "", dataBnc$RAT2, perl=T)
dataBnc$RATX <- gsub("\\?", "", dataBnc$RATX, perl=T)

stringof3codes <- rep(NA, times=nrow(dataBnc))
for (i in 1:nrow(dataBnc)) {
	row <- t(dataBnc[i,c("RAT1", "RAT2", "RATX")])
	temp <- sort(as.vector(row))
	stringof3codes[i] <- paste(temp, collapse="-")
}
table(stringof3codes)

# FOR TABLE 5 IN PAPER:
table(dataBnc$MFA.NorG, stringof3codes)
round(100*prop.table(table(dataBnc$MFA.NorG, stringof3codes), 2), 1)

# Set the majority rules code as a column
dataBnc$MAJCODE <- as.factor(stringof3codes)
levels(dataBnc$MAJCODE) <- c("G", "G", "N", "N")

# Get speaker social factors from the speaker labels
dataBnc$Corpus <- as.factor(gsub("^(\\w\\w\\w).*$", "\\1", dataBnc$Spkr, perl=T))
dataBnc$Gender <- as.factor(gsub("^.*?_([mf])_\\d+$", "\\1", dataBnc$Spkr, perl=T))
levels(dataBnc$Gender) <- c("F", "F", "M")
dataBnc$AgeGrp <- as.factor(gsub("^.*?_(ag\\d)_.*$", "\\1", dataBnc$Spkr, perl=T))
levels(dataBnc$AgeGrp) <- c("AG1", "AG2", "AG3", "AG4", "AG3")
dataBnc$AgeGrp <- relevel(dataBnc$AgeGrp, ref="AG4")
dataBnc$SEC <- as.factor(gsub("^.*?_(se\\d)_.*$", "\\1", dataBnc$Spkr, perl=T))
levels(dataBnc$SEC) <- c("SE2+3", "SE1", "SE2+3", "SE2+3")
dataBnc$SEC <- relevel(dataBnc$SEC, ref = "SE1")

# Add word length in syllables, using this syllable counting function from Kendall (2013)
source("http://lingtools.uoregon.edu/scripts/english_syllable_counter-102.R")
dataBnc$SyllCnt <- NA
for (word in unique(dataBnc$Word)) {
	dataBnc$SyllCnt[dataBnc$Word==word] <- english_syllable_count(word)
}

# Set the dependent vars for statistical analysis
dataBnc$MAJisIN <- dataBnc$MAJCODE=="N"
dataBnc$MFAisIN <- dataBnc$MFA.NorG=="N"

# Need this package, install.packages(...) first if you don't have it
library(lmerTest)


mod.I.1 <- glmer(MAJisIN ~ Corpus + Gender + AgeGrp + SEC + SyllCnt + (1|Spkr) + (1|Word), data=dataBnc, family="binomial", control=glmerControl(optimizer="bobyqa"))
summary(mod.I.1)

mod.II.1 <- glmer(MFAisIN ~ Corpus + Gender + AgeGrp + SEC + SyllCnt + (1|Spkr) + (1|Word), data=dataBnc, family="binomial", control=glmerControl(optimizer="bobyqa"))
summary(mod.II.1)

# And then add Grammatical Category, in a simply binary format
# MODELS I AND II FOR PAPER:

# e.g. Recode the POS tags to grammatical categories for (ING) analysis
table(dataBnc$POS)
dataBnc$GramCat <- "V-like"
dataBnc$GramCat[dataBnc$POS %in% c("ADJ", "N", "NPR", "NS")] <- "N-like" # includes ADJs
dataBnc$GramCat[dataBnc$POS %in% c("Q+N", "P", "INTJ")] <- "N-like" # e.g. everything, during 
dataBnc$GramCat[dataBnc$Word %in% c("something", "nothing", "fucking", "motherfucking")] <- "V-like"
table(dataBnc$GramCat, dataBnc$POS)
dataBnc$GramCat <- as.factor(dataBnc$GramCat)
dataBnc$GramCat <- relevel(dataBnc$GramCat, ref="V-like")

mod.I.2 <- glmer(MAJisIN ~ Corpus + Gender + AgeGrp + SEC + GramCat + SyllCnt + (1|Spkr) + (1|Word), data=dataBnc, family="binomial", control=glmerControl(optimizer="bobyqa"))
summary(mod.I.2)

mod.II.2 <- glmer(MFAisIN ~ Corpus + Gender + AgeGrp + SEC + GramCat + SyllCnt + (1|Spkr) + (1|Word), data=dataBnc, family="binomial", control=glmerControl(optimizer="bobyqa"))
summary(mod.II.2)

# FOR TABLE 1:
#  print out individual speakers' rates of -in:
prop.IN <- function(x) { length(which(x=="N"))/length(x) }
spkr.percents <- round(100*tapply(dataBnc$MAJCODE, dataBnc$Spkr, prop.IN), 1)
sort(spkr.percents)

# Now merge Dataset B with the MFCC data

# Load MFCC dataset in
ccdat <- read.delim(paste(wk.dir, "CORAALDC_IN+ING_Words_wMFCCs_Dec2020.txt", sep=""))

# It contains non-variable IN words (e.g. begin, in), and ING words (e.g. thing, wing)
#  and variable (ING) words (e.g. talking, walking, something)
table(ccdat$Class)

ccdat$Corpus <- gsub("^(\\w\\w\\w)_.*$", "\\1", ccdat$spkr, perl=T)
table(ccdat$Corpus)

ccvar <- droplevels(ccdat[ccdat$Class=="VAR",])

dataBmfcc <- NA
first <- TRUE
missd <- NA
mfrst <- TRUE
for (sp in unique(dataBnc$Spkr)) {
	adataBnc <- droplevels(dataBnc[dataBnc$Spkr==sp,])
	accvar <- droplevels(ccvar[as.character(ccvar$spkr)==as.character(sp),])
	j <- 1
	for (i in 1:nrow(adataBnc)) {
		
		match.ind <- which(accvar$word==as.character(adataBnc$Word[i]) & accvar$vwl.st > adataBnc$StT[i] & accvar$vwl.st < adataBnc$EnT[i])
		if (length(match.ind)==1) {
			ctemp <- cbind(adataBnc[i,], accvar[match.ind,])
			
			if (first) {
				dataBmfcc <- ctemp
				first <- FALSE
			} else {
				dataBmfcc <- rbind(dataBmfcc, ctemp)
			}
		} else {
			cat("\n", i, ": Failed to match one MFCC entry for ", sp, ", ", as.character(adataBnc$Word[i]), ". Matched ", length(match.ind), ".", sep="")
			mtemp <- adataBnc[i,]
			mtemp$Matches <- length(match.ind)
			if (mfrst) {
				missd <- mtemp
				mfrst <- FALSE
			} else {
				missd <- rbind(missd, mtemp)
			}
		}
	}
}
dim(dataBmfcc)
dim(missd) # 4!

# Inspect
head(dataBmfcc)

# how many "missed" (ING)s are shorter than 0.05 s?
dim(missd[missd$ing.dur <= 0.05,]) # = 4, all of them

dim(dataBmfcc)

write.table(dataBmfcc, file=paste(wk.dir, "DatasetB_wMFCCs.txt", sep=""), quote=F, sep="\t", row.names=F)

# Last load Dataset C and merge those data with MFCCs
dCint <- read.delim(paste(wk.dir, "DatasetC_Initial.txt", sep=""))

dataCmfcc <- NA
first <- TRUE
missd <- NA
mfrst <- TRUE
for (sp in unique(dCint$Spkr)) {
	adataCnc <- droplevels(dCint[dCint$Spkr==sp,])
	accvar <- droplevels(ccvar[as.character(ccvar$spkr)==as.character(sp),])
	j <- 1
	for (i in 1:nrow(adataCnc)) {
		
		match.ind <- which(accvar$word==as.character(adataCnc$Word[i]) & accvar$vwl.st > adataCnc$StT[i] & accvar$vwl.st < adataCnc$EnT[i])
		if (length(match.ind)==1) {
			ctemp <- cbind(adataCnc[i,], accvar[match.ind,])
			
			if (first) {
				dataCmfcc <- ctemp
				first <- FALSE
			} else {
				dataCmfcc <- rbind(dataCmfcc, ctemp)
			}
		} else {
			cat("\n", i, ": Failed to match one MFCC entry for ", sp, ", ", as.character(adataCnc$Word[i]), ". Matched ", length(match.ind), ".", sep="")
			mtemp <- adataCnc[i,]
			mtemp$Matches <- length(match.ind)
			if (mfrst) {
				missd <- mtemp
				mfrst <- FALSE
			} else {
				missd <- rbind(missd, mtemp)
			}
		}
	}
}
dim(dataCmfcc)
dim(missd) # 0

# Inspect
head(dataCmfcc)

write.table(dataCmfcc, file=paste(wk.dir, "DatasetC_wMFCCs.txt", sep=""), quote=F, sep="\t", row.names=F)
