# c_classify_handcoded_datasets.R
# Kendall et al. Performance in Coding Sociolinguistic Variables (Frontiers)
#
# Code trains classifiers on the hand-coded Dataset B and tests them
#  on Datasets B and C. Models III and IV from paper, and other table material are
#  generated here.
#
# To run this code, you need the DatasetB_wMFCCs.txt and DatasetC_wMFCCs.txt files
#  which are provided in the Supplementary Materials with the paper and can be
#  created using different settings from the a_... and b_... R files included.
#

# Set up graphical defaults
par(bg="white")
defaults <- par(no.readonly=TRUE)

# Load libraries, install.packages(...) first if you don't have them
library(e1071)
library(randomForest)
library(pROC)
library(lmerTest)

# Set this directory to the location of the files
wk.dir <- "~/Documents/CORAAL/ING/Supplementary Materials/"

# Read in Dataset B following its integration with the extracted MFCCs (see code b_...)
datasetB <- read.delim(paste(wk.dir, "DatasetB_wMFCCs.txt", sep=""))

trmdat <- droplevels(datasetB[!is.na(datasetB$X1_CepCoef_25),])

trn.matr <- trmdat[,grep("CepCoef", names(trmdat), perl=T)]
trn.labs <- relevel(trmdat$MAJCODE, ref="N")

# Many tokens in Dataset B didn't have MFCC entries, 803 do
length(trn.labs)

# Inspect, rates of -in use
table(trn.labs)
round(100*prop.table(table(trn.labs)), 1)

# Tune the models first, using 10 fold cross-validation
#  these are stochastic, so exact results will vary
# For the Dataset B analysis we'll just use the resultant parameters to set later 
#  models, for the Dataset C analysis we'll use these as our models.
# Note we haven't found much evidence that the different costs and gammas change outcomes
#  much, but still is best to empirical tune the models for the data.

lin.tuned <- tune(svm, trn.matr, trn.labs, kernel="linear", ranges = list(cost = 2^(0:6)), tunecontrol = tune.control(sampling="cross", cross=10, best.model=T))
lin.tuned
# best cost is often 2 (but moves a bit)

rbf.tuned <- tune(svm, trn.matr, trn.labs, kernel="radial", ranges = list(gamma = 2^(-10:1), cost = 2^(0:6)), tunecontrol = tune.control(sampling="cross", cross=10, best.model=T))
rbf.tuned
# also changes a bit but gamma = 0.015625 & cost = 2 are common

rf.tuned <- tuneRF(trn.matr, trn.labs, plot=F, doBest=T)
rf.tuned # best mtry often = 12


# Set folds randomly for 10-fold cross-validation
set.seed(201220) # make random algorithm repeatable
ran.nums <- rep(1:10, length=length(trn.labs))
trn.flds <- sample(ran.nums)

# Now do cross-fold hold out tests.
#  This will iterate over each fold, training and testing on folds of Dataset B.
#  This long list of variables are set up to store the various outputs from the runs.
trn.results.lin <- c()
tst.results.lin <- c()
trn.results.rbf <- c()
tst.results.rbf <- c()
trn.results.rf <- c()
tst.results.rf <- c()
trn.act.in <- c()
tst.act.in <- c()
trn.pred.in.lin <- c()
tst.pred.in.lin <- c()
trn.pred.in.rbf <- c()
tst.pred.in.rbf <- c()
trn.pred.in.rf <- c()
tst.pred.in.rf <- c()
trn.l.cm.list <- list()
trn.r.cm.list <- list()
trn.f.cm.list <- list()
tst.l.cm.list <- list()
tst.r.cm.list <- list()
tst.f.cm.list <- list()
lin.tuned.list <- list()
rbf.tuned.list <- list()
rf.tuned.list <- list()
tst.lin.roc.list <- list()
tst.rbf.roc.list <- list()
tst.rf.roc.list <- list()
tst.lin.aucs <- c()
tst.rbf.aucs <- c()
tst.rf.aucs <- c()
for (i in sort(unique(trn.flds))) {
	cat("\nHolding out fold", i, "...", sep=" ")
	
	# hold out random 10% for initial testing
	ho.indices <- which(trn.flds==i)
	
	rtrn.matr <- trn.matr[-ho.indices,]
	rtrn.labs <- trn.labs[-ho.indices]
	
	ho.matr <- trn.matr[ho.indices,]
	ho.labs <- trn.labs[ho.indices]
	
	# Train the models, using parameters from tuning above (comment out if tuning each pass)
	#  Nb. These are set to common tuning recommendations from above, but the values
	#  vary. They don't appear to change the results very much though.
	lin.svm <- svm(rtrn.matr, rtrn.labs, type="C-classification", kernel="linear", cost=2, probability=T)
	rbf.svm <- svm(rtrn.matr, rtrn.labs, type="C-classification", kernel="radial", gamma=2^-6, cost=2, probability=T)
	ranfor <- randomForest(rtrn.matr, rtrn.labs, mtry=12)
	
	# Uncomment if you want to tune the models for each iteration. However, 
	#  tuning each pass is much slower and does not appear to lead to improved models.
	 
	# lin.tuned <- tune(svm, rtrn.matr, rtrn.labs, type="C-classification", kernel="linear", ranges = list(cost = 2^(0:6)), tunecontrol = tune.control(sampling="cross", cross=10, best.model=T), probability=T)
	# lin.tuned.list[[i]] <- lin.tuned
	# rbf.tuned <- tune(svm, rtrn.matr, rtrn.labs, kernel="radial", ranges = list(gamma = 2^(-10:1), cost = 2^(0:6)), tunecontrol = tune.control(sampling="cross", cross=10, best.model=T), probability=T)
	# rbf.tuned.list[[i]] <- rbf.tuned
	# rf.tuned <- tuneRF(rtrn.matr, rtrn.labs, plot=F, doBest=T)
	# rf.tuned.list[[i]] <- rf.tuned
	
	# # Use best models from tuning
	# lin.svm <- lin.tuned$best.model
	# rbf.svm <- rbf.tuned$best.model
	# ranfor <- rf.tuned

	# How does fitted SVM model do on the training data?
	lin.fitd <- as.character(fitted(lin.svm))
	rbf.fitd <- as.character(fitted(rbf.svm))
	rf.fitd <- as.character(ranfor$predicted)
	# The diagonal of this table are the correct labels
	act.labs <- as.character(rtrn.labs)
	(trn.l.cm.list[[i]] <- lin.cm <- table(act.labs, lin.fitd))
	(trn.r.cm.list[[i]] <- rbf.cm <- table(act.labs, rbf.fitd))
	(trn.f.cm.list[[i]] <- rf.cm <- table(act.labs, rf.fitd))
	# % accuracy:
	(trn.results.lin[i] <- round(100*sum(diag(lin.cm))/length(rtrn.labs), 1))
	(trn.results.rbf[i] <- round(100*sum(diag(rbf.cm))/length(rtrn.labs), 1))
	(trn.results.rf[i] <- round(100*sum(diag(rf.cm))/length(rtrn.labs), 1))
	# Percent of actual -ins 
	(trn.act.in[i] <- round(100*length(which(act.labs=="N"))/length(act.labs), 1))
	# Percent of -in predictions 
	(trn.pred.in.lin[i] <- round(100*length(which(lin.fitd=="N"))/length(lin.fitd), 1))
	(trn.pred.in.rbf[i] <- round(100*length(which(rbf.fitd=="N"))/length(rbf.fitd), 1))
	(trn.pred.in.rf[i] <- round(100*length(which(rf.fitd=="N"))/length(rf.fitd), 1))
	
	# Now we use it to classify (predict the labels of) the test speakers
	ho.l.preds <- predict(lin.svm, ho.matr)
	ho.r.preds <- predict(rbf.svm, ho.matr)
	ho.f.preds <- predict(ranfor, ho.matr)
	
	# How did it do?... and store
	(tst.l.cm.list[[i]] <- tst.l.cm <- table(ho.labs, ho.l.preds))
	(tst.r.cm.list[[i]] <- tst.r.cm <- table(ho.labs, ho.r.preds))
	(tst.f.cm.list[[i]] <- tst.f.cm <- table(ho.labs, ho.f.preds))
	# % accuracy:
	(tst.results.lin[i] <- round(100*sum(diag(tst.l.cm))/length(ho.labs), 1))
	(tst.results.rbf[i] <- round(100*sum(diag(tst.r.cm))/length(ho.labs), 1))
	(tst.results.rf[i] <- round(100*sum(diag(tst.f.cm))/length(ho.labs), 1))
	# Percent of actual -ins 
	(tst.act.in[i] <- round(100*length(which(ho.labs=="N"))/length(ho.labs), 1))
	# Percent of -in predictions 
	(tst.pred.in.lin[i] <- round(100*length(which(ho.l.preds=="N"))/length(ho.l.preds), 1))
	(tst.pred.in.rbf[i] <- round(100*length(which(ho.r.preds=="N"))/length(ho.r.preds), 1))
	(tst.pred.in.rf[i] <- round(100*length(which(ho.f.preds=="N"))/length(ho.f.preds), 1))

	# Calculate AUC pROC roc()
	tst.lin.roc <- roc(as.numeric(ho.labs),as.numeric(ho.l.preds), quiet=T)
	tst.lin.roc.list[[i]] <- tst.lin.roc
	(tst.lin.aucs[i] <- auc(tst.lin.roc))	
	tst.rbf.roc<-roc(as.numeric(ho.labs),as.numeric(ho.r.preds), quiet=T)
	tst.rbf.roc.list[[i]] <- tst.rbf.roc
	(tst.rbf.aucs[i] <- auc(tst.rbf.roc))
	tst.rf.roc<-roc(as.numeric(ho.labs),as.numeric(ho.f.preds), quiet=T)
	tst.rf.roc.list[[i]] <- tst.rf.roc
	(tst.rf.aucs[i] <- auc(tst.rf.roc))
}
# Can inspect the accuracy from each run
trn.results.lin
tst.results.lin
trn.results.rbf
tst.results.rbf
trn.results.rf
tst.results.rf

# And examine the results in various ways
mean(trn.results.lin)
mean(trn.results.rbf)
mean(trn.results.rf)

range(tst.results.lin)
range(tst.results.rbf)
range(tst.results.rf)

tst.act.in
range(tst.act.in)
mean(tst.act.in)
sd(tst.act.in)

mean(tst.pred.in.lin)
sd(tst.pred.in.lin)

mean(tst.pred.in.rbf)
sd(tst.pred.in.rbf)

mean(tst.pred.in.rf)
sd(tst.pred.in.rf)

# Examine results more systematically, FOR TABLE 7 IN THE PAPER:
# Linear SVM results on Dataset B
mean(tst.results.lin)
sd(tst.results.lin)

# Calculate precision (True Positives/(True Positives + False Positives))
TPs.lin <- sapply(tst.l.cm.list, "[[", 1)
FPs.lin <- sapply(tst.l.cm.list, "[[", 3)
prec.lin <- TPs.lin/(TPs.lin+FPs.lin)
round(mean(prec.lin),2)
round(sd(prec.lin),2)
# Calculate recall (TPs/TPs+False Negatives)
FNs.lin <- sapply(tst.l.cm.list, "[[", 2)
recall.lin <- TPs.lin/(TPs.lin+FNs.lin)
round(mean(recall.lin),2)
round(sd(recall.lin),2)

(f.score.lin <- 2*((prec.lin*recall.lin)/prec.lin+recall.lin))
round(mean(f.score.lin), 2)
round(sd(f.score.lin), 2)

round(mean(tst.lin.aucs), 2)
round(sd(tst.lin.aucs), 2)

# Radial SVM results on Dataset B
mean(tst.results.rbf)
sd(tst.results.rbf)

# Calculate precision (True Positives/(True Positives + False Positives))
TPs.rbf <- sapply(tst.r.cm.list, "[[", 1)
FPs.rbf <- sapply(tst.r.cm.list, "[[", 3)
prec.rbf <- TPs.rbf/(TPs.rbf+FPs.rbf)
round(mean(prec.rbf),2)
round(sd(prec.rbf),2)
# Calculate recall (TPs/TPs+False Negatives)
FNs.rbf <- sapply(tst.r.cm.list, "[[", 2)
recall.rbf <- TPs.rbf/(TPs.rbf+FNs.rbf)
round(mean(recall.rbf),2)
round(sd(recall.rbf),2)

(f.score.rbf <- 2*((prec.rbf*recall.rbf)/prec.rbf+recall.rbf))
round(mean(f.score.rbf), 2)
round(sd(f.score.rbf), 2)

round(mean(tst.rbf.aucs), 2)
round(sd(tst.rbf.aucs), 2)


# Random Forest results on Dataset B
mean(tst.results.rf)
sd(tst.results.rf)

# Calculate precision (True Positives/(True Positives + False Positives))
TPs.rf <- sapply(tst.f.cm.list, "[[", 1)
FPs.rf <- sapply(tst.f.cm.list, "[[", 3)
prec.rf <- TPs.rf/(TPs.rf+FPs.rf)
round(mean(prec.rf),2)
round(sd(prec.rf),2)
# Calculate recall (TPs/TPs+False Negatives)
FNs.rf <- sapply(tst.f.cm.list, "[[", 2)
recall.rf <- TPs.rf/(TPs.rf+FNs.rf)
round(mean(recall.rf),2)
round(sd(recall.rf),2)

(f.score.rf <- 2*((prec.rf*recall.rf)/prec.rf+recall.rf))
round(mean(f.score.rf), 2)
round(sd(f.score.rf), 2)

round(mean(tst.rf.aucs), 2)
round(sd(tst.rf.aucs), 2)

# Plot predications and accuracies, FIGURE 1 IN THE PAPER
# Uncomment to save the plot to disk (and uncomment dev.off() at bottom)
# out.file <- paste(wk.dir, "DatasetB-training+testing-results.jpg", sep="")
# jpeg(filename = out.file, width = 5, height = 3.5, units = "in", res = 300, pointsize = 9, quality = 100, bg = "white", type = "quartz")
par(mai=c(0.7, 0.7, 0.1, 0.1), family="Helvetica")
plot(trn.results.lin, typ="l", ylim=c(50, 104), col="#9ecae1", ylab="Percent", xlab="Fold", lty=2, lwd=2)
lines(trn.act.in, col="darkgray", lty=6, lwd=2)
lines(tst.act.in, col="black", lty=6, lwd=2)
lines(tst.results.lin, col="#3182bd", lwd=2)
lines(trn.pred.in.lin, col="#9ecae1", lty=3)
lines(tst.pred.in.lin, col="#3182bd", lty=4)
lines(trn.results.rbf, col="#fdae6b", lty=2, lwd=2)
lines(tst.results.rbf, col="#e6550d", lwd=2)
lines(trn.pred.in.rbf, col="#fdae6b", lty=3)
lines(tst.pred.in.rbf, col="#e6550d", lty=4)
lines(trn.results.rf, col="#a1d99b", lty=2, lwd=2)
lines(tst.results.rf, col="#31a354", lwd=2)
lines(trn.pred.in.rf, col="#a1d99b", lty=3)
lines(tst.pred.in.rf, col="#31a354", lty=4)
legend("bottomleft", c("Train Accuracy", "Test Accuracy", "Train % PRED -in", "Test % PRED -in"), ncol=2, cex=0.4, col=c("#9ecae1", "#3182bd", "#9ecae1", "#3182bd"), lty=c(2, 1, 3, 4), lwd=c(2, 2, 1, 1), bty="n", title="Linear SVM")
legend("bottom", c("Train Accuracy", "Test Accuracy", "Train % PRED -in", "Test % PRED -in"), ncol=2, cex=0.4, col=c("#fdae6b", "#e6550d", "#fdae6b", "#e6550d"), lty=c(2, 1, 3, 4), lwd=c(2, 2, 1, 1), bty="n", title="Radial SVM")
legend("bottomright", c("Train Accuracy", "Test Accuracy", "Train % PRED -in", "Test % PRED -in"), ncol=2, cex=0.4, col=c("#a1d99b", "#31a354", "#a1d99b", "#31a354"), lty=c(2, 1, 3, 4), lwd=c(2, 2, 1, 1), bty="n", title="Random Forest")
legend("topright", c("Train % -in", "Test % -in"), ncol=1, cex=0.4, col=c("darkgray", "black"), lty=c(6, 6), lwd=c(2, 2), bty="n", title="ACTUAL")
legend("topleft", "Classifier performance on Dataset B", cex=0.8, bty="n")
par(defaults)
# dev.off()

# Can plot additional measures, e.g. AUCs
# Uncomment to save the plot to disk (and uncomment dev.off() at bottom)
# out.file<-paste(wk.dir, "DatasetB-training+testing-AUCs.jpg", sep="")
# jpeg(filename = out.file, width = 5, height = 3.5, units = "in", res = 300, pointsize = 9, quality = 100, bg = "white", type = "quartz")
par(mai=c(0.7, 0.7, 0.1, 0.1), family="Helvetica")
plot(tst.lin.aucs, typ="l", ylim=c(0.5, 1), col="#3182bd", lwd=2, lty=3, ylab="AUC", xlab="Fold")
lines(tst.rbf.aucs, col="#e6550d", lwd=2, lty=2)
lines(tst.rf.aucs, col="#31a354", lwd=2, lty=1)
legend("bottom", c("Linear SVM", "Radial SVM", "Random Forest"), ncol=3, cex=0.8, col=c("#3182bd", "#e6550d", "#31a354"), lty=c(3, 2, 1), lwd=c(2, 2, 2), bty="n")
legend("topleft", "Classifier AUC on Dataset B", cex=0.8, bty="n")
par(defaults)
# dev.off()

# Now test the 900 tokens coded by two analysts in Dataset C

# Read in Dataset C (which is already combined with MFCCs )
datasetC <- read.delim(paste(wk.dir, "DatasetC_wMFCCs.txt", sep=""))

# Trim out tokens without MFCC values
dCtest <- droplevels(datasetC[!is.na(datasetC$X1_CepCoef_25),])
dim(dCtest) # = 580 have MFCCs

# Set majority rules code, though since only two analysts we'll toss out disagreements
dCtest$MAJCODE <- "?"
dCtest$MAJCODE[dCtest$Coder1.NorG=="N" & dCtest$Coder2.NorG=="N"] <- "N"
dCtest$MAJCODE[dCtest$Coder1.NorG=="G" & dCtest$Coder2.NorG=="G"] <- "G"

table(dCtest$MAJCODE)
  # ?   G   N 
 # 64 205 311 

# drop the 64 tokens that had disagreement
dCtest <- droplevels(dCtest[dCtest$MAJCODE %in% c("N", "G"),])
dim(dCtest) # 516

dCtest$MAJCODE <- as.factor(dCtest$MAJCODE)

# Set up feature matrix and labels
dC.matr <- dCtest[,grep("CepCoef", names(dCtest), perl=T)]
dC.labs <- relevel(dCtest$MAJCODE, ref="N")

# Much of what follows is FOR TABLE 7 IN PAPER

# Inspect rate of -in use
round(100*prop.table(table(dC.labs)), 1)

dC.lin <- svm(trn.matr, trn.labs, kernel="linear", cost=2, probability=T)
dC.lin.preds <- predict(dC.lin, dC.matr, probability=T)

dC.rbf <- svm(trn.matr, trn.labs, kernel="radial", cost=2, gamma=2^-6, probability=T)
dC.rbf.preds <- predict(dC.rbf, dC.matr, probability=T)

dC.rf <- randomForest(trn.matr, trn.labs, mtry=12)
dC.rf.preds <- predict(dC.rf, dC.matr)


(dC.l.cmatr <- table(dC.labs, dC.lin.preds))
# Accuracy
(dC.l.acc <- round(100*sum(diag(dC.l.cmatr))/length(dC.labs), 1))
# Percent of -in predictions 
(dC.l.predin <- round(100*length(which(dC.lin.preds =="N"))/length(dC.lin.preds), 1))
# Calculate precision (True Positives/(True Positives + False Positives))
dC.l.TPs <- dC.l.cmatr[1]
dC.l.FPs <- dC.l.cmatr[3]
dC.l.PREC<- dC.l.TPs/(dC.l.TPs + dC.l.FPs)
round(dC.l.PREC, 2)
# Calculate recall (TPs/TPs+False Negatives)
dC.l.FNs <- dC.l.cmatr[2]
dC.l.RECL<- dC.l.TPs/(dC.l.TPs + dC.l.FNs)
round(dC.l.RECL, 2)
# F1 Score
dC.l.fsc <- 2*((dC.l.PREC* dC.l.RECL)/dC.l.PREC + dC.l.RECL)
round(dC.l.fsc, 2)
# AUC
dC.l.roc <- roc(as.numeric(dC.labs),as.numeric(dC.lin.preds))
round(dC.l.roc$auc, 2)

(dC.r.cmatr <- table(dC.labs, dC.rbf.preds))
# Accuracy
(dC.r.acc <- round(100*sum(diag(dC.r.cmatr))/length(dC.labs), 1))
# Percent of -in predictions 
(dC.r.predin <- round(100*length(which(dC.rbf.preds =="N"))/length(dC.rbf.preds), 1))
# Calculate precision (True Positives/(True Positives + False Positives))
dC.r.TPs <- dC.r.cmatr[1]
dC.r.FPs <- dC.r.cmatr[3]
dC.r.PREC<- dC.r.TPs/(dC.r.TPs + dC.r.FPs)
round(dC.r.PREC, 2)
# Calculate recall (TPs/TPs+False Negatives)
dC.r.FNs <- dC.r.cmatr[2]
dC.r.RECL<- dC.r.TPs/(dC.r.TPs + dC.r.FNs)
round(dC.r.RECL, 2)
# F1 Score
dC.r.fsc <- 2*((dC.r.PREC* dC.r.RECL)/dC.r.PREC + dC.r.RECL)
round(dC.r.fsc, 2)
# AUC
dC.r.roc <- roc(as.numeric(dC.labs),as.numeric(dC.rbf.preds))
round(dC.r.roc$auc, 2)

(dC.f.cmatr <- table(dC.labs, dC.rf.preds))
# Accuracy
(dC.f.acc <- round(100*sum(diag(dC.f.cmatr))/length(dC.labs), 1))
# Percent of -in predictions 
(dC.f.predin <- round(100*length(which(dC.rf.preds =="N"))/length(dC.rf.preds), 1))
# Calculate precision (True Positives/(True Positives + False Positives))
dC.f.TPs <- dC.f.cmatr[1]
dC.f.FPs <- dC.f.cmatr[3]
dC.f.PREC<- dC.f.TPs/(dC.f.TPs + dC.f.FPs)
round(dC.f.PREC, 2)
# Calculate recall (TPs/TPs+False Negatives)
dC.f.FNs <- dC.f.cmatr[2]
dC.f.RECL<- dC.f.TPs/(dC.f.TPs + dC.f.FNs)
round(dC.f.RECL, 2)
# F1 Score
dC.f.fsc <- 2*((dC.f.PREC* dC.f.RECL)/dC.f.PREC + dC.f.RECL)
round(dC.f.fsc, 2)
# AUC
dC.f.roc <- roc(as.numeric(dC.labs),as.numeric(dC.rf.preds))
round(dC.f.roc$auc, 2)

# Get social factors for statistical modeling
dCtest$Gender <- as.factor(gsub("^.*?_([mf])_\\d+$", "\\1", dCtest$Spkr, perl=T))
levels(dCtest$Gender) <- c("F", "F", "M")
dCtest$AgeGrp <- as.factor(gsub("^.*?_(ag\\d)_.*$", "\\1", dCtest$Spkr, perl=T))
levels(dCtest$AgeGrp) <- c("AG1", "AG2", "AG3", "AG4", "AG3")
dCtest$AgeGrp <- relevel(dCtest$AgeGrp, ref="AG4")
dCtest$SEC <- as.factor(gsub("^.*?_(se\\d)_.*$", "\\1", dCtest$Spkr, perl=T))
levels(dCtest$SEC) <- c("SE2", "SE1", "SE2", "SE3")
dCtest$SEC <- relevel(dCtest$SEC, ref = "SE1")

# Add word length in syllables, using this syllable counting function from Kendall (2013)
source("http://lingtools.uoregon.edu/scripts/english_syllable_counter-102.R")
dCtest$SyllCnt <- NA
for (word in unique(dCtest$Word)) {
	dCtest$SyllCnt[dCtest$Word==word] <- english_syllable_count(word)
}


# Now model the data:

dCtest$MAJisIN <- dCtest$MAJCODE=="N"
mod.III.1 <- glmer(MAJisIN ~ Corpus + Gender + AgeGrp + SEC + SyllCnt + (1|Spkr) + (1|Word), data=dCtest, family="binomial", control=glmerControl(optimizer="bobyqa"))
summary(mod.III.1)

# Since Radial SVM did best, we just model the output from that one:

dCtest$RBFisIN <- dC.rbf.preds=="N"
mod.IV.1 <- glmer(RBFisIN ~ Corpus + Gender + AgeGrp + SEC + SyllCnt + (1|Spkr) + (1|Word), data=dCtest, family="binomial", control=glmerControl(optimizer="bobyqa"))
summary(mod.IV.1)

# FOR TABLE 8 IN THE PAPER (include Grammatical Category, in simplified binary factor) 

# E.g. recode the POS tags to grammatical categories for (ING) analysis
table(dCtest$POS)
dCtest$GramCat <- "V-like"
dCtest$GramCat[dCtest$POS %in% c("ADJ", "N", "NPR", "NS")] <- "N-like" # includes ADJs
dCtest$GramCat[dCtest$POS %in% c("Q+N", "P", "INTJ")] <- "N-like" # e.g. everything, during 
dCtest$GramCat[dCtest$Word %in% c("something", "nothing", "fucking", "motherfucking")] <- "V-like"
table(dCtest$GramCat, dCtest$POS)
dCtest$GramCat <- as.factor(dCtest$GramCat)
dCtest$GramCat <- relevel(dCtest$GramCat, ref="V-like")

mod.III.2 <- glmer(MAJisIN ~ Corpus + Gender + AgeGrp + SEC + GramCat + SyllCnt + (1|Spkr) + (1|Word), data=dCtest, family="binomial", control=glmerControl(optimizer="bobyqa"))
summary(mod.III.2)


mod.IV.2 <- glmer(RBFisIN ~ Corpus + Gender + AgeGrp + SEC + GramCat + SyllCnt + (1|Spkr) + (1|Word), data=dCtest, family="binomial", control=glmerControl(optimizer="bobyqa"))
summary(mod.IV.2)
