# d_classify_using_nonvar_training.R
# Kendall et al. Performance in Coding Sociolinguistic Variables (Frontiers)
#
# Code trains classifiers on the non-variable data provided in the Supplementary 
#  Materials in CORAALDC_IN+ING_Words_wMFCCs_Dec2020.txt file (or that you
#  create using a_... R script, and tests them on Datasets B and C. 
#  Models V and VI from paper, and other table material are generated here.
#
# To run this code, you need the CORAALDC_IN+ING_Words_wMFCCs_Dec2020.txt file
#  as well as the DatasetB_wMFCCs.txt and DatasetC_wMFCCs.txt files which
#  are provided in the Supplementary Materials with the paper and can be
#  created using different settings from the a_... and b_... R files included.
#

# Load necessary libraries (install.packages(...) first, if you don't have them)
library(e1071)
library(randomForest)
library(pROC)
library(lmerTest)

# Set wk.dir to where the data are
wk.dir <- "~/Documents/CORAAL/ING/Supplementary Materials/"

# Read in the dataset with MFCCs for many, but not all, tokens
ccdat <- read.delim(paste(wk.dir, "CORAALDC_IN+ING_Words_wMFCCs_Dec2020.txt", sep=""))

# Can inspect the data, e.g. (uncomment if desired)
# wdclass <- unique(ccdat[,c("word", "Class")])
# split(wdclass$word, wdclass$Class)

# Dataset includes categorical IN words (e.g. begin, in), ING words (e.g. thing, wing)
#  and variable (ING) words (e.g. talking, working, something)
table(ccdat$Class)

# If interested in breakdown by corpus...
ccdat$Corpus <- gsub("^(\\w\\w\\w)_.*$", "\\1", ccdat$spkr, perl=T)
table(ccdat$Corpus)

# Trim the data to remove all of the tokens which don't have MFCCs
trdat <- droplevels(ccdat[!is.na(ccdat$X1_CepCoef_25),])
dim(trdat)

# Trim out all of the DCA interviewers & misc speakers
trdat <- droplevels(trdat[-grep("DCA_int", trdat$spkr, perl=T),])
trdat <- droplevels(trdat[trdat$spkr!="misc",])
dim(trdat)

# Counts by word type
table(trdat$Class)

# Create a version that only includes the categorical, i.e. non-variable, words
cat.dat <- droplevels(trdat[trdat$Class!="VAR",])

# and to inspect just the variable (ING) words
v.dat <- droplevels(trdat[trdat$Class=="VAR",])
dim(v.dat)


# The original data creation included -s final words (e.g. things)
#  Drop these words for the categorical tokens
cat.dat <- droplevels(cat.dat[-grep(".*s$", cat.dat$word, perl=T),])

# Final counts as reported in the paper
dim(cat.dat)
table(cat.dat$Class)

# Now subsample the categorical words to make the classes balanced
#  ING words are the limiting class, so we need to subsample IN words
table(cat.dat$Class)

# Make random algorithm repeatable
set.seed(122020)

# Now set up a new data.frame with a random, balanced set of tokens
sin.dat <- cat.dat[cat.dat$Class=="IN",]
sing.dat <- cat.dat[cat.dat$Class=="ING",]
sub.dat <- rbind(sin.dat[sample(1:nrow(sin.dat), nrow(sing.dat)),], sing.dat)
table(sub.dat$Class)

# Load in Dataset B
datasetB <- read.delim(paste(wk.dir, "DatasetB_wMFCCs.txt", sep=""))
dim(datasetB) # = 1131

# Trim Dataset B to only contain tokens with MFCC values
trdatB <- droplevels(datasetB[!is.na(datasetB$X1_CepCoef_25),])
dim(trdatB) # = 803

# Set up the feature matrix and labels for classification
var.matr <- trdatB[,grep("CepCoef", names(trdatB), perl=T)]
var.labs <- trdatB$MAJCODE

# Set the labels to match up with the cat.dat factors for the classes
levels(var.labs) <- c("ING", "IN")
var.labs <- as.factor(as.character(var.labs))

# Set the training data to be the matrix and labels from the sub.dat categorical data
trn.matr <- droplevels(sub.dat[,grep("CepCoef", names(sub.dat), perl=T)])
trn.labs <- sub.dat$Class

length(trn.labs)

# Results from below FOR TABLE 9 IN PAPER

table(var.labs)
round(100*prop.table(table(var.labs)), 1) # Dataset B = 62.4% -in

# Train the linear SVM, tuning using 10 fold cross-validation
lin.tuned <- tune(svm, trn.matr, trn.labs, kernel="linear", ranges = list(cost = 2^(0:6)), tunecontrol = tune.control(sampling="cross", cross=10), best.model=T)
# Inspect the tuning
lin.tuned
# Use the best model from tuning
cat.lin <- lin.tuned$best.model

# Now predict the variable tokens in Dataset B.
#  And calculate the various performance measures
lin.preds <- predict(cat.lin, var.matr)

# Confusion matrix
(l.cmatr <- table(var.labs, lin.preds))
# Accuracy
(l.acc <- round(100*sum(diag(l.cmatr))/length(var.labs), 1))
# Percent of -in predictions 
(l.predin <- round(100*length(which(lin.preds =="IN"))/length(lin.preds), 1))
# Calculate precision (True Positives/(True Positives + False Positives))
l.TPs <- l.cmatr[1]
l.FPs <- l.cmatr[3]
l.PREC<- l.TPs/(l.TPs + l.FPs)
round(l.PREC, 2)
# Calculate recall (TPs/TPs+False Negatives)
l.FNs <- l.cmatr[2]
l.RECL<- l.TPs/(l.TPs + l.FNs)
round(l.RECL, 2)
# Calculate F1 Score
l.fsc <- 2*((l.PREC* l.RECL)/l.PREC + l.RECL)
round(l.fsc, 2)
# And area under the ROC curve
l.roc <- roc(as.numeric(var.labs),as.numeric(lin.preds))
round(l.roc$auc, 2)

# Then train and tune the radial SVM.
rbf.tuned <- tune(svm, trn.matr, trn.labs, kernel="radial", ranges = list(gamma = 2^(-10:1), cost = 2^(0:6)), tunecontrol = tune.control(sampling="cross", cross=10), best.model=T)
# Inspect the outcome of tuning
rbf.tuned
# Save the best model from tuning
cat.rbf <- rbf.tuned$best.model

# And test predictions on Dataset B
rbf.preds <- predict(cat.rbf, var.matr)
(r.cmatr <- table(var.labs, rbf.preds))
# Accuracy
(r.acc <- round(100*sum(diag(r.cmatr))/length(var.labs), 1))
# Percent of -in predictions 
(r.predin <- round(100*length(which(rbf.preds =="IN"))/length(rbf.preds), 1))
# Calculate precision (True Positives/(True Positives + False Positives))
r.TPs <- r.cmatr[1]
r.FPs <- r.cmatr[3]
r.PREC<- r.TPs/(r.TPs + r.FPs)
round(r.PREC, 2)
# Calculate recall (TPs/TPs+False Negatives)
r.FNs <- r.cmatr[2]
r.RECL<- r.TPs/(r.TPs + r.FNs)
round(r.RECL, 2)
# F1 Score
r.fsc <- 2*((r.PREC* r.RECL)/r.PREC + r.RECL)
round(r.fsc, 2)
# AUC
r.roc <- roc(as.numeric(var.labs),as.numeric(rbf.preds))
round(r.roc$auc, 2)

# Train and tune the Random Forest
rf.tuned <- tuneRF(trn.matr, trn.labs, plot=F, doBest=T)
# The output is the best model, but rename to match the others
(cat.rf <- rf.tuned)

# Then predict and calculate performance metrics
rf.preds <- predict(cat.rf, var.matr)
(f.cmatr <- table(var.labs, rf.preds))
# Accuracy
(f.acc <- round(100*sum(diag(f.cmatr))/length(var.labs), 1))
# Percent of -in predictions 
(f.predin <- round(100*length(which(rf.preds =="IN"))/length(rf.preds), 1))
# Calculate precision (True Positives/(True Positives + False Positives))
f.TPs <- f.cmatr[1]
f.FPs <- f.cmatr[3]
f.PREC<- f.TPs/(f.TPs + f.FPs)
round(f.PREC, 2)
# Calculate recall (TPs/TPs+False Negatives)
f.FNs <- f.cmatr[2]
f.RECL<- f.TPs/(f.TPs + f.FNs)
round(f.RECL, 2)
# F1 Score
f.fsc <- 2*((f.PREC* f.RECL)/f.PREC + f.RECL)
round(f.fsc, 2)
# AUC
f.roc <- roc(as.numeric(var.labs),as.numeric(rf.preds))
round(f.roc$auc, 2)

# Now compare the human coded Dataset B and classified Dataset B, via regressions
dBtest <- trdatB[,-grep("CepCoef", names(trdatB), perl=T)]
levels(dBtest$Gender)
levels(dBtest$AgeGrp)
dBtest$AgeGrp <- relevel(dBtest$AgeGrp, ref="AG4")
levels(dBtest$SEC)
dBtest$SEC <- relevel(dBtest$SEC, ref="SE1")


# Examine through statistical modeling - Dataset B modeled here although not
#  included in the paper (Dataset C modeled below is)

# Recode the POS tags to grammatical categories for (ING) analysis
table(dBtest$POS)
dBtest$GramCat <- "V-like"
dBtest$GramCat[dBtest$POS %in% c("ADJ", "N", "NPR", "NS")] <- "N-like" # includes ADJs
dBtest$GramCat[dBtest$POS %in% c("Q+N", "P", "INTJ")] <- "N-like" # e.g. everything, during 
dBtest$GramCat[dBtest$Word %in% c("something", "nothing", "fucking", "motherfucking")] <- "V-like"
table(dBtest$GramCat, dBtest$POS)
dBtest$GramCat <- as.factor(dBtest$GramCat)
dBtest$GramCat <- relevel(dBtest$GramCat, ref="V-like")

# Model human coded data
mod.y.2 <- glmer(MAJisIN ~ Corpus + Gender + AgeGrp + SEC + GramCat + SyllCnt + (1|Spkr) + (1|Word), data=dBtest, family="binomial", control=glmerControl(optimizer="bobyqa"))
summary(mod.y.2)

# Model the output from the Radial SVM
dBtest$RBFisIN <- rbf.preds=="IN"
mod.z.2 <- glmer(RBFisIN ~ Corpus + Gender + AgeGrp + SEC + GramCat + SyllCnt + (1|Spkr) + (1|Word), data=dBtest, family="binomial", control=glmerControl(optimizer="bobyqa"))
summary(mod.z.2)

# As a second test, consider how the classifiers perform on Dataset C
datasetC <- read.delim(paste(wk.dir, "DatasetC_wMFCCs.txt", sep=""))

# Trim Dataset C to only contain tokens with MFCC values
dCtest <- droplevels(datasetC[!is.na(datasetC$X1_CepCoef_25),])
dim(dCtest) #580

# Set majority rules code
dCtest$MAJCODE <- "?"
dCtest$MAJCODE[dCtest$Coder1.NorG=="N" & dCtest$Coder2.NorG=="N"] <- "N"
dCtest$MAJCODE[dCtest$Coder1.NorG=="G" & dCtest$Coder2.NorG=="G"] <- "G"

table(dCtest$MAJCODE)
  # ?   G   N 
 # 64 205 311 

# Drop the 64 tokens that had disagreement
dCtest <- droplevels(dCtest[dCtest$MAJCODE %in% c("N", "G"),])
dim(dCtest) # 516

# Rates of -in
round(100*prop.table(table(dCtest$MAJCODE)), 1) # Actual rates = 60.3%

# Set up feature matrix and labels
dCtest$MAJCODE <- as.factor(dCtest$MAJCODE) 
dC.matr <- dCtest[,grep("CepCoef", names(dCtest), perl=T)]
dC.labs <- dCtest$MAJCODE
levels(dC.labs) <- c("ING", "IN")
dC.labs <- relevel(dC.labs, ref="IN")

# Generate predictions from linear SVM and calculate performance metrics
dC.lin.preds <- predict(cat.lin, dC.matr)
(dC.l.cmatr <- table(dC.labs, dC.lin.preds))
# Accuracy
(dC.l.acc <- round(100*sum(diag(dC.l.cmatr))/length(dC.labs), 1))
# Percent of -in predictions 
(dC.l.predin <- round(100*length(which(dC.lin.preds =="IN"))/length(dC.lin.preds), 1))
# Calculate precision (True Positives/(True Positives + False Positives))
dC.l.TPs <- dC.l.cmatr[1]
dC.l.FPs <- dC.l.cmatr[3]
dC.l.PREC<- dC.l.TPs/(dC.l.TPs + dC.l.FPs)
round(dC.l.PREC, 2)
# Calculate recall (TPs/TPs+False Negatives)
dC.l.FNs <- dC.l.cmatr[2]
dC.l.RECL<- dC.l.TPs/(dC.l.TPs + dC.l.FNs)
round(dC.l.RECL, 2)
# F1 Score
dC.l.fsc <- 2*((dC.l.PREC* dC.l.RECL)/dC.l.PREC + dC.l.RECL)
round(dC.l.fsc, 2)
# AUC
dC.l.roc <- roc(as.numeric(dC.labs),as.numeric(dC.lin.preds))
round(dC.l.roc$auc, 2)

# Generate predictions from radial SVM and calculate performance metrics
dC.rbf.preds <- predict(cat.rbf, dC.matr)
(dC.r.cmatr <- table(dC.labs, dC.rbf.preds))
# Accuracy
(dC.r.acc <- round(100*sum(diag(dC.r.cmatr))/length(dC.labs), 1))
# Percent of -in predictions 
(dC.r.predin <- round(100*length(which(dC.rbf.preds =="IN"))/length(dC.rbf.preds), 1))
# Calculate precision (True Positives/(True Positives + False Positives))
dC.r.TPs <- dC.r.cmatr[1]
dC.r.FPs <- dC.r.cmatr[3]
dC.r.PREC<- dC.r.TPs/(dC.r.TPs + dC.r.FPs)
round(dC.r.PREC, 2)
# Calculate recall (TPs/TPs+False Negatives)
dC.r.FNs <- dC.r.cmatr[2]
dC.r.RECL<- dC.r.TPs/(dC.r.TPs + dC.r.FNs)
round(dC.r.RECL, 2)
# F1 Score
dC.r.fsc <- 2*((dC.r.PREC* dC.r.RECL)/dC.r.PREC + dC.r.RECL)
round(dC.r.fsc, 2)
# AUC
dC.r.roc <- roc(as.numeric(dC.labs),as.numeric(dC.rbf.preds))
round(dC.r.roc$auc, 2)

# Generate predictions from Random Forest and calculate performance metrics
dC.rf.preds <- predict(cat.rf, dC.matr)
(dC.f.cmatr <- table(dC.labs, dC.rf.preds))
# Accuracy
(dC.f.acc <- round(100*sum(diag(dC.f.cmatr))/length(dC.labs), 1))
# Percent of -in predictions 
(dC.f.predin <- round(100*length(which(dC.rf.preds =="IN"))/length(dC.rf.preds), 1))
# Calculate precision (True Positives/(True Positives + False Positives))
dC.f.TPs <- dC.f.cmatr[1]
dC.f.FPs <- dC.f.cmatr[3]
dC.f.PREC<- dC.f.TPs/(dC.f.TPs + dC.f.FPs)
round(dC.f.PREC, 2)
# Calculate recall (TPs/TPs+False Negatives)
dC.f.FNs <- dC.f.cmatr[2]
dC.f.RECL<- dC.f.TPs/(dC.f.TPs + dC.f.FNs)
round(dC.f.RECL, 2)
# F1 Score
dC.f.fsc <- 2*((dC.f.PREC* dC.f.RECL)/dC.f.PREC + dC.f.RECL)
round(dC.f.fsc, 2)
# AUC
dC.f.roc <- roc(as.numeric(dC.labs),as.numeric(dC.rf.preds))
round(dC.f.roc$auc, 2)

# Then, model these outcomes using regression

dCtest$Gender <- as.factor(gsub("^.*?_([mf])_\\d+$", "\\1", dCtest$Spkr, perl=T))
levels(dCtest$Gender) <- c("F", "F", "M")
dCtest$AgeGrp <- as.factor(gsub("^.*?_(ag\\d)_.*$", "\\1", dCtest$Spkr, perl=T))
levels(dCtest$AgeGrp) <- c("AG1", "AG2", "AG3", "AG4", "AG3")
dCtest$AgeGrp <- relevel(dCtest$AgeGrp, ref="AG4")
dCtest$SEC <- as.factor(gsub("^.*?_(se\\d)_.*$", "\\1", dCtest$Spkr, perl=T))
levels(dCtest$SEC) <- c("SE2", "SE1", "SE2", "SE3")
dCtest$SEC <- relevel(dCtest$SEC, ref = "SE1")

# add word length in syllables
source("http://lingtools.uoregon.edu/scripts/english_syllable_counter-102.R")
dCtest$SyllCnt <- NA
for (word in unique(dCtest$Word)) {
	dCtest$SyllCnt[dCtest$Word==word] <- english_syllable_count(word)
}

# Examine through statistical modeling - Dataset B modeled here although not
#  included in the paper (Dataset C modeled below is)

# Recode the POS tags to grammatical categories for (ING) analysis
table(dCtest$POS)
dCtest$GramCat <- "V-like"
dCtest$GramCat[dCtest$POS %in% c("ADJ", "N", "NPR", "NS")] <- "N-like" # includes ADJs
dCtest$GramCat[dCtest$POS %in% c("Q+N", "P", "INTJ")] <- "N-like" # e.g. everything, during 
dCtest$GramCat[dCtest$Word %in% c("something", "nothing", "fucking", "motherfucking")] <- "V-like"
table(dCtest$GramCat, dCtest$POS)
dCtest$GramCat <- as.factor(dCtest$GramCat)
dCtest$GramCat <- relevel(dCtest$GramCat, ref="V-like")

# FOR TABLE 10 IN PAPER 

# Model the human coded data (this replicates Model III from paper)
dCtest$MAJisIN <- dCtest$MAJCODE=="N"
mod.iii.again <- glmer(MAJisIN ~ Corpus + Gender + AgeGrp + SEC + GramCat + SyllCnt + (1|Spkr) + (1|Word), data=dCtest, family="binomial", control=glmerControl(optimizer="bobyqa"))
summary(mod.iii.again)

# Model the output from the radial SVM
dCtest$RBFisIN <- dC.rbf.preds=="IN"
mod.v.2 <- glmer(RBFisIN ~ Corpus + Gender + AgeGrp + SEC + GramCat + SyllCnt + (1|Spkr) + (1|Word), data=dCtest, family="binomial", control=glmerControl(optimizer="bobyqa"))
summary(mod.v.2)


